!======================================================================  
!===>  PROGRAM CALCULATES THE AVERAGE GRAIN SIZE IN THE PRSENCE OF ^^^^
!===>  GROWING PRECIPITATE (CARBIDES/NITRIDES) ************************
!======================================================================  
!===>  GLOBAL DECLARATIONS ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^  
!======================================================================  
MODULE GLOBAL  
IMPLICIT NONE  
  
INTEGER,PARAMETER :: LL=49  
INTEGER :: L1,L2  
REAL*8 :: G0,M0,QG,QP,R,C5
REAL,DIMENSION(LL) :: DT,T  
  
END MODULE GLOBAL  
!======================================================================  
!===>  PROGRAM MAIN   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^  
!======================================================================  
PROGRAM MAIN  
USE GLOBAL  
IMPLICIT NONE  
  
INTEGER :: I  
  
!INITIALIZE KINETIC CONSTANTS AND OTHER INPUT PARAMETERS  
      L1=LL        ;    L2=L1-1	   ;    T(:)=0.      ;      DT(:)=0.0  
      G0=10.0      ;    M0=20.0E9   ;    QG=224.0E3   ;      R=8.31432      
  	  QP=240.0E3   ;    C5=6.67E4  

!INPUT DATA READS HERE (TIME VS TEMPERATURE)  *************************    
      OPEN(UNIT=25,FILE="ags3.in")  
          DO I=1,L1   
             READ(25,*)DT(I),T(I)  
          END DO  
      CLOSE(25)   
!  
    
      CALL GRAIN_SIZE  
      WRITE(*,*)"PROGRAM TERMINATED"  
END PROGRAM MAIN  
!======================================================================  
!===>  SUBROUTINE GRAIN_SIZE CALCULATES AUSTENITE GRAIN SIZE FOR THE 
!===>  GIVEN TEMPERATURE WITH TIME  ***********************************
!======================================================================  
SUBROUTINE GRAIN_SIZE  
USE GLOBAL   
IMPLICIT NONE  
INTEGER :: ISTART,IEND,IVAL  
REAL*8 :: DT1,DT2,GLIM,GVALUE,TIMEAT,TIMESTEP  
REAL*8 :: K1,K2,K3,K4,KAVER  
REAL*8 :: G0LIM,RADIUS_PRECIP,VOLFRA_PRECIP,ZENER_COEFF    
REAL*8 :: PSTRENGTH,PVALUE    
REAL*8,DIMENSION(LL) :: G  
REAL*8,EXTERNAL :: FUNCT_GCHANGE  
      
!
	  ZENER_COEFF=0.5    
      RADIUS_PRECIP=0.014    
      VOLFRA_PRECIP=1.4E-4    
      G0LIM=ZENER_COEFF*(RADIUS_PRECIP/VOLFRA_PRECIP) 
!  
      G(:)=G0   !INITIALIZATION 
      OPEN(UNIT=30,FILE="ags3out.m")    
          WRITE(30,*)"g=["  
          DO IVAL=2,L2  
            DT1=DT(IVAL-1)   
            DT2=DT(IVAL)  
            TIMEAT=DT1  
            TIMESTEP=DT2-DT1  
! STARTING AND ENDING DATA POINTS FOR HEATING/COOLING PERIODS 
! ISTART/IEND FOR HEATING AND COOLING PERIODS SHOULD BE GIVEN CORRECTLY
! ELSE RESULTS IN DIFFERENT TEMPERATURES DURING INTERPOLATION.  
            IF(IVAL.LE.23)THEN	   
              !FOR HEATING PERIOD  ***** 
              ISTART=2  
              IEND=23 
            ELSE  
              !FOR COOLING PERIOD  ***** 
              ISTART=24  
              IEND=L2      
            END IF  
!            
			CALL SIMPSON(ISTART,IEND,DT1,DT2,PVALUE)    
            PSTRENGTH=C5*PVALUE     
            GLIM=(G0LIM**3+(ZENER_COEFF/VOLFRA_PRECIP)**3 &
		        & *PSTRENGTH)**(1./3.)    
!
            GVALUE=G(IVAL-1) 
!FOURTH ORDER RUNGE-KUTTA METHOD USED TO FIND THE RATE OF CHANGE OF 
!AVERAGE GRAIN SIZE.             
            K1=TIMESTEP*FUNCT_GCHANGE(ISTART,IEND,TIMEAT,GLIM,GVALUE)    
            K2=TIMESTEP*FUNCT_GCHANGE(ISTART,IEND,(TIMEAT+TIMESTEP/2) &    
              & ,GLIM,(GVALUE+K1/2))    
            K3=TIMESTEP*FUNCT_GCHANGE(ISTART,IEND,(TIMEAT+TIMESTEP/2) &  
              & ,GLIM,(GVALUE+K2/2))  
            K4=TIMESTEP*FUNCT_GCHANGE(ISTART,IEND,(TIMEAT+TIMESTEP) &  
              & ,GLIM,(GVALUE+K3))  
            KAVER=(K1+2.*K2+2.*K3+K4)/6  
            G(IVAL)=G(IVAL-1)+KAVER  
            WRITE(30,*)DT(IVAL),G(IVAL)  
          END DO             
        WRITE(30,*) "];"  
        WRITE(30,*) "plot(g(:,1),g(:,2))"  !MATLAB PLOT
      CLOSE(30)      
END SUBROUTINE GRAIN_SIZE  
!======================================================================    
!===>  SIMPSONS 1/3 RULE USED TO EVALUATE AN INTEGRAL OF A FUNCTION  **
!======================================================================    
SUBROUTINE SIMPSON(ISTART,IEND,DT1,DT2,VALUE)    
USE GLOBAL    
IMPLICIT NONE    
    
INTEGER :: I,N    
INTEGER,INTENT(IN) :: ISTART,IEND    
REAL*8,INTENT(IN) :: DT1,DT2    
REAL*8 :: DT3,H,S0,S1,S2    
REAL*8,INTENT(OUT) :: VALUE    
REAL*8,EXTERNAL :: FUNCT    
    
!INITIALISE VARIABLES:    
       N = 5    
       S0 = 0.0    
       S1 = 0.0    
       S2 = 0.0    
       VALUE=0.0    
       H=(DT2-DT1)/(2*N)      
       S0=FUNCT(ISTART,IEND,DT1)+FUNCT(ISTART,IEND,DT2)    
       DO I=1,(2*N-1),2    
         DT3=DT1+I*H    
         S1=S1+FUNCT(ISTART,IEND,DT3)    
       END DO    
       DO I=2,(2*N-2),2    
         DT3=DT1+I*H    
         S2=S2+FUNCT(ISTART,IEND,DT3)    
       END DO    
       VALUE=(H/3)*(S0+4*S1+2*S2)    
    
END SUBROUTINE SIMPSON    
!======================================================================               
FUNCTION FUNCT(ISTART,IEND,DT3)    
USE GLOBAL    
IMPLICIT NONE    
    
INTEGER,INTENT(IN) :: ISTART,IEND    
REAL*8,INTENT(IN) :: DT3    
REAL*8 :: TVALUE,FUNCT    
REAL*8,EXTERNAL :: FUNCT_NDDI    
    
      TVALUE=FUNCT_NDDI(ISTART,IEND,DT3)    
      FUNCT=EXP(-QP/(R*TVALUE))/TVALUE    
    
END FUNCTION FUNCT        
!======================================================================    
FUNCTION FUNCT_GCHANGE(ISTART,IEND,TIMEAT,GLIM,GVALUE)    
USE GLOBAL  
IMPLICIT NONE  
  
INTEGER,INTENT(IN) :: ISTART,IEND  
REAL*8,INTENT(IN) :: TIMEAT,GLIM,GVALUE  
REAL*8 :: TN,TVALUE  
REAL*8 :: FUNCT_GCHANGE  
REAL*8,EXTERNAL :: FUNCT_NDDI  
	  
	  TN=0.5 !TIME EXPONENT    
      TVALUE=FUNCT_NDDI(ISTART,IEND,TIMEAT)  
      FUNCT_GCHANGE=M0*EXP(-QG/(R*TVALUE))*(1/GVALUE-1/GLIM)**(1/TN-1)  
  
END FUNCTION FUNCT_GCHANGE  
!======================================================================
!===>  FUNCTION FUNCT_NDDI INTERPOLATION SCHEME USED TO FIND THE TEMPE-
!===>  RATURE AT INTERMEDIATE TIMES, USED IN RUNGE-KUTTA METHOD   *****
!======================================================================
FUNCTION FUNCT_NDDI(ISTART,IEND,DT3)  
USE GLOBAL  
IMPLICIT NONE  
  
INTEGER,INTENT(IN) :: ISTART,IEND  
INTEGER :: I,J,K,IMAX  
REAL*8,INTENT(IN) :: DT3  
REAL*8 :: FACTOR  
REAL*8,DIMENSION(LL) :: POLY  
REAL*8,DIMENSION(LL,LL) :: DD  
REAL*8 :: FUNCT_NDDI  
  
!INITIALISATION  
      IMAX=(IEND-ISTART+1)  
      DD(:,:)=0.0     

!FOR FIRST DIVIDED DIFFERENCE   
      DO I=ISTART,(IMAX-1),1    
        DD(1,I)=(T(I+1)-T(I))/(DT(I+1)-DT(I))  
      END DO  
  
!FOR SECOND DIFFERENCE ONWARDS        
      DO J=(ISTART+1),(IMAX-1),1   
        DO I=1,(IMAX-J),1  
          DD(J,I)=(DD(J-1,I+1)-DD(J-1,I))/(DT(J+I)-DT(I))  
        END DO  
      END DO  
  
!FOR POLYNOMIALS  
      POLY(ISTART)=T(ISTART)  
      DO K=(ISTART+1),IMAX,1  
        FACTOR=1.0  
        DO I=1,K-1,1  
          FACTOR=FACTOR*(DT3-DT(I))  
        END DO  
        POLY(K)=POLY(K-1)+FACTOR*DD(K-1,1)  
        FUNCT_NDDI=POLY(K)  
      END DO      
     
END FUNCTION FUNCT_NDDI  
!======================================================================  
!===>  END OF THE CODE   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^  
!======================================================================  
